package com.indigitall.indigitall_flutter_plugin.utils

import android.app.PendingIntent
import android.app.TaskStackBuilder
import android.content.Context
import android.content.Intent
import com.indigitall.android.commons.models.CoreAction
import com.indigitall.android.commons.models.CorePush
import com.indigitall.android.commons.models.CorePushAction
import com.indigitall.android.commons.utils.CorePreferenceUtils
import com.indigitall.android.commons.utils.IntentUtils
import com.indigitall.android.push.models.Push
import com.indigitall.android.push.services.StatisticService
import com.indigitall.indigitall_flutter_plugin.activities_ind.FTIndigitallHiddenActivity

object FTIndigitallIntentUtils {
    fun getIntent(
        context: Context?,
        push: CorePush,
        action: CorePushAction?,
        clickedButton: Int
    ): PendingIntent? {
        try {
            action?.let {
                val intent = context?.let { it1 -> action.getIntent(it1) }
                if (intent != null) {
                    if (action.type == CoreAction.Type.APP) {
                        intent.putExtra(Push.EXTRA_PUSH, push.toString())
                    }
                    val indigitallIntent = Intent(
                        context,
                        FTIndigitallHiddenActivity::class.java
                    )
                    indigitallIntent.flags = Intent.FLAG_ACTIVITY_NEW_TASK
                    indigitallIntent.putExtra(
                        StatisticService.EXTRA_APP_KEY,
                        CorePreferenceUtils.getAppKey(context)
                    )
                    indigitallIntent.putExtra(StatisticService.EXTRA_PUSH_ID, push.id)
                    indigitallIntent.putExtra(StatisticService.EXTRA_CLICKED_BUTTON, clickedButton)
                    indigitallIntent.putExtra(StatisticService.EXTRA_INTENT_ACTION, intent)

                    indigitallIntent.putExtra(StatisticService.EXTRA_SENDING_ID, push.sendingId)
                    indigitallIntent.putExtra(StatisticService.EXTRA_CAMPAIGN_ID, push.campaignId)

                    if (push.journeyStateId != 0)
                        indigitallIntent.putExtra(
                            StatisticService.EXTRA_JOURNEY_STATE_ID,
                            push.journeyStateId
                        )

                    if (push.cjCurrentStateId != 0)
                        indigitallIntent.putExtra(
                            StatisticService.EXTRA_CJ_CURRENT_STATE_ID,
                            push.cjCurrentStateId
                        )
                    if (push.journeyExecution != 0)
                        indigitallIntent.putExtra(
                            StatisticService.EXTRA_JOURNEY_EXECUTION,
                            push.journeyExecution
                        )

                    indigitallIntent.putExtra(Push.EXTRA_PUSH, push.toString())
                    action.topics?.let { topics ->
                        if (topics.isNotEmpty()) {
                            indigitallIntent.putExtra(
                                StatisticService.EXTRA_ACTION_TOPICS,
                                action.topicsToString()
                            )
                        }
                    }

                    val stackBuilder = TaskStackBuilder.create(context)
                    stackBuilder.addNextIntentWithParentStack(indigitallIntent)
                    return if (action.isDestroy()) {
                        stackBuilder.getPendingIntent(
                            push.id * 10 + clickedButton,
                            IntentUtils.getPendingIntentFlags(PendingIntent.FLAG_UPDATE_CURRENT)
                        )
                    } else {
                        PendingIntent.getActivity(
                            context,
                            0,
                            Intent(),
                            IntentUtils.getPendingIntentFlags(0)
                        )
                    }
                }
            }

        } catch (e: Exception) {
            e.printStackTrace()
        }
        return null
    }
}