//
//  IndigitallInAppFlutterPlugin.m
//  indigitall_flutter_plugin
//
//  Created by indigitall on 18/1/23.
//

#import "IndigitallInAppFlutterPlugin.h"

@implementation IndigitallInAppFlutterPlugin

NSString *INAPP_PLUGIN_TOPICS_LIST = @"inAppTopicsList";
NSString *INAPP_PLUGIN_TOPICS_SUBSCRIBE = @"inAppTopicsSubscribe";
NSString *INAPP_PLUGIN_TOPICS_UNSUBSCRIBE = @"inAppTopicsUnsubscribe";

+ (void) getInAppModelWithParams:(NSObject *)params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSString *inAppCode;
        NSString *appKey = [INDefaults getAppKey];
        NSString *domainInApp = [INInAppDefaults getDomainInApp];
        NSMutableDictionary *dicParams = [NSMutableDictionary new];
        if ([params isKindOfClass:[NSString class]]) {
            inAppCode = (NSString*)params;
        } else {
            dicParams = (NSMutableDictionary *)params;
            if (dicParams[@"inAppCode"]) inAppCode = dicParams[@"inAppCode"];
            if (dicParams[@"appKey"]) appKey = dicParams[@"appKey"];
            if (dicParams[@"domainInApp"]) domainInApp = dicParams[@"domainInApp"];
        }
        
        INInAppConfig *config = [[INInAppConfig alloc]initWithAppKey:appKey];
        config.domainInApp = domainInApp;
        [InAppIndigitall showCustomInAppWithConfig:config inAppCode:inAppCode onSuccess:^(INInApp * _Nonnull inApp){
            [IndigitallInAppFParse jsonFromInApp:inApp withResult:result];
        } onFailed:^(INInAppError * _Nonnull error) {
            [IndigitallInAppFParse jsonFromError:error withResult:result];
        }];
    });
}

+ (void) inAppGetHtmlWithUrl: (NSDictionary *)params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        INInApp *inApp = [[INInApp alloc]initWithJson:params];
        if (inApp.properties.InAppContentUrl != nil) {
            [[NSURLSession.sharedSession dataTaskWithURL:inApp.properties.InAppContentUrl completionHandler:^(NSData * _Nullable data, NSURLResponse * _Nullable response, NSError * _Nullable error) {
                if (data == nil){
                    NSLog(@"inAppGetHtml data: %@", error.localizedDescription);
                    return;
                }
                NSHTTPURLResponse *httpResponse = (NSHTTPURLResponse *) response;
                if (httpResponse.statusCode != 200){
                    NSLog(@"inAppGetHtml httpResponse.statusCode: %@", error.localizedDescription);
                    return;
                }
                NSString *content = [[NSString alloc] initWithData:data encoding:NSUTF8StringEncoding];
                if (content == nil){
                    NSLog(@"inAppGetHtml content: %@", error.localizedDescription);
                    return;
                }
                
                NSString *contentHtml = content;
                if (inApp.customData != nil) {
                    for (int i = 0; i < inApp.customData.count; i++) {
                        contentHtml = [contentHtml stringByReplacingOccurrencesOfString:[NSString stringWithFormat:@"{{%@}}", [inApp.customData allKeys][i]] withString:[inApp.customData allValues][i]];
                    }
                }
                NSString *htmlWithoutOnClick = [contentHtml stringByReplacingOccurrencesOfString:@"InAppIndigitall.onClick" withString:@"IndigitallInApp.onClick"];
                NSString *htmlWiThJS = [NSString stringWithFormat:@"%@%@", htmlWithoutOnClick, INInAppAPIConstants.INAPP_SCRIPT ];
                result(htmlWiThJS);
                
            }]resume];
        } else {
            NSLog(@"inAppGetHtml: No url on InApp object");
        }
    });
}

+ (void) showPopUpWithParams:(NSDictionary *)params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        NSString *inAppCode;
        BOOL closeIconDisabled = false;
        NSString *closeIconName;
        BOOL closePopupWhenClicked = false;
        UIButton *button;
        if (params[@"popUpCode"]) inAppCode = params[@"popUpCode"];
        if ([params[@"closeIconDisabled"]boolValue] ==  true) closeIconDisabled = true;
        if (params[@"imageCloseButtonName"]){
            closeIconName = params[@"imageCloseButtonName"];
            button = [[UIButton alloc]init];
            [button setImage:[UIImage imageNamed:closeIconName] forState:normal];
            [button setTranslatesAutoresizingMaskIntoConstraints:false];
        }
        NSString *appKey;
        NSString *domainInApp = @"https://device-api.indigitall.com/v2";
        if (params[@"appKey"]) appKey = params[@"appKey"];
        if (appKey == nil) appKey = [INInAppDefaults getAppKey];
        if (params[@"domainInApp"]) domainInApp = params[@"domainInApp"];
        if (params[@"closePopupWhenClicked"]) closePopupWhenClicked = params[@"closePopupWhenClicked"];
        
        [self inAppCreateDataWithAppKey: appKey];
        INInAppConfig *config = [[INInAppConfig alloc]init];
        config.appKey = appKey;
        config.domainInApp = domainInApp;
        config.closePopupWhenClicked = closePopupWhenClicked;
        [INInAppDefaults setDomainInApp:domainInApp];
        [InAppIndigitall showPopupWithConfig:config popupId:inAppCode closeIcon:button closeIconDisabled:closeIconDisabled didAppear:^{
            result(@"");
        } didCancel:^{
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:@"didCancel" withResult:result];
        } didTouchWithAction:^(INInAppAction * _Nonnull action) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:@"didTouchWithAction" withResult:result];
        } didDismissed:^{
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:@"didDismissed" withResult:result];
        } onShowTimeFinished:^(INInApp * _Nonnull inApp, int showTime) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:[NSString stringWithFormat:@"onShowTimeFinished after %d seconds", showTime] withResult:result];
        } didExpired:^(INInApp * _Nonnull inApp, INError * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error.message withResult:result];
        } didShowManyTimes:^(INInApp * _Nonnull inApp, INError * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error.message withResult:result];
        } didClickOut:^(INInApp * _Nonnull inApp, INError * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error.message withResult:result];
        } didDismissForever:^(INInApp * _Nonnull inApp, INError * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error.message withResult:result];
        } failed:^(INError * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error.message withResult:result];
        } formFailed:^(NSArray<INInAppError *> * _Nonnull error) {
            [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_POPUP_ERROR" withMessage:error[0].message withResult:result];
        } formSubmit:^(INInApp * _Nonnull inApp) {
            [IndigitallInAppFParse jsonFromInApp:inApp withResult:result];
        }];
    });
}

+ (void) inAppFormSubmit:(NSArray *) params
                  result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (params != nil && params.count > 0) {
            id object = params[0];
            INInApp *inApp = [[INInApp alloc]initWithJson:object];
            if (inApp != nil) {
                NSMutableDictionary *dicForms = [NSMutableDictionary new];
                [dicForms setValue:params[1] forKey:@"formOptions"];
                [dicForms setValue:params[2] forKey:@"jsonForm"];
                [dicForms setValue:params[3] forKey:@"jsonCustomer"];
                NSArray<INInAppError *> *errorList = [INInAppFormUtils checkFieldsForm:[dicForms copy] withInApp:inApp];
                if (errorList.count > 0) {
                    [IndigitallInAppFParse setErrorWithErrorCode:@"INAPP_FORM_ERROR" withMessage:errorList[0].message withResult:result];
                } else {
                    result(@"");
                }
            }
        }
    });
}

+ (void) onClickWithParams: (NSDictionary *) params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        INInApp *inApp = [[INInApp alloc]initWithJson:params];
        [INInAppUtils inAppWasTappedWithInApp:inApp];
    });
}

+ (void) eventPrintInAppRequestWithParams: (NSDictionary *) params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        INInApp *inApp = [[INInApp alloc]initWithJson:params];
        INEventRequestInApp *request = [[INEventRequestInApp alloc]initWithInApp:inApp];
        [INInAppAPIClient postEventPrintRequest:request completion:[[INBaseCallback alloc]init]];
    });
}

+ (void) eventClickInAppRequestWithParams: (NSDictionary *) params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        INInApp *inApp = [[INInApp alloc]initWithJson:params];
        INEventRequestInApp *request = [[INEventRequestInApp alloc]initWithInApp:inApp];
        [INInAppAPIClient postEventClickRequest:request completion:[[INBaseCallback alloc]init]];
    });
}

+ (void) topicsListWithParams: (NSDictionary *) params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        [InAppIndigitall topicsListWithOnSuccess:^(NSArray<INInAppTopic *> * _Nonnull topics) {
            [IndigitallInAppFParse jsonArrayFromtopics:topics withKey:INAPP_PLUGIN_TOPICS_LIST withResult:result];
        } onError:^(INError * _Nonnull error) {
            [IndigitallInAppFParse jsonFromError:error withResult:result];
        }];
    });
}

+ (void) topicsSubscribeWithParams: (NSArray<NSString*>*)params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (params != nil) {
            [InAppIndigitall topicsSubscribeWithArrayString:params onSuccess:^(NSArray<NSString *> * _Nonnull topics) {
                [IndigitallInAppFParse resultFromTopicString:topics withKey:INAPP_PLUGIN_TOPICS_SUBSCRIBE withResult:result];
            } onError:^(INError * _Nonnull error) {
                [IndigitallInAppFParse jsonFromError:error withResult:result];
            }];
        }
    });
}

+ (void) topicsUnsubscribeWithParams: (NSArray<NSString*>*)params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        if (params != nil) {
            [InAppIndigitall topicsUnSubscribeWithArrayString:params onSuccess:^(NSArray<NSString *> * _Nonnull topics) {
                [IndigitallInAppFParse resultFromTopicString:topics withKey:INAPP_PLUGIN_TOPICS_UNSUBSCRIBE withResult:result];
            } onError:^(INError * _Nonnull error) {
                [IndigitallInAppFParse jsonFromError:error withResult:result];
            }];
        }
    });
}

+ (void) addNewInAppToDismissForeverWithParams: (NSDictionary *) params result: (FlutterResult _Nonnull)result {
    dispatch_async(dispatch_get_main_queue(), ^{
        INInApp *inApp = [[INInApp alloc]initWithJson:params];
        NSURL *url;
       if (inApp != nil) {
           [INInAppUtils addNewInAppToDismissForever:inApp];
       }
    });
}

+ (void) inAppCreateDataWithAppKey: (NSString *)appKeyValue {
    NSString *appKey = appKeyValue;
    if (appKeyValue == nil) appKey = [INInAppDefaults getAppKey];
    dispatch_async(dispatch_get_main_queue(), ^{
        NSString *deviceId = [INInAppDefaults getDeviceId];
        if (deviceId == nil) {
            deviceId = [INEventUtils generateUUID];
            [INInAppDefaults setDeviceId:deviceId];
        }
        if ([INInAppDefaults getExternalId] == nil && appKey != nil ) {
            NSString *code = [[[INHmacUtils alloc]init] hmacsha256:deviceId  secret:appKey];
            [INInAppDefaults setExternalId:code];
        }
    });
}

@end
